@@ -1,7 +1,7 @@
 /*
  * qrencode - QR Code encoder
  *
- * Copyright (C) 2006-2012 Kentaro Fukuchi <kentaro@fukuchi.org>
+ * Copyright (C) 2006-2017 Kentaro Fukuchi <kentaro@fukuchi.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@
 #include "mqrspec.h"
 #include "bitstream.h"
 #include "qrinput.h"
-#include "rscode.h"
+#include "rsecc.h"
 #include "split.h"
 #include "mask.h"
 #include "mmask.h"
@@ -59,14 +59,14 @@ typedef struct {
 	int count;
 } QRRawCode;
 
-static void RSblock_initBlock(RSblock *block, int dl, unsigned char *data, int el, unsigned char *ecc, RS *rs)
+static void RSblock_initBlock(RSblock *block, int dl, unsigned char *data, int el, unsigned char *ecc)
 {
 	block->dataLength = dl;
 	block->data = data;
 	block->eccLength = el;
 	block->ecc = ecc;
 
-	encode_rs_char(rs, data, ecc);
+	RSECC_encode(dl, el, data, ecc);
 }
 
 static int RSblock_init(RSblock *blocks, int spec[5], unsigned char *data, unsigned char *ecc)
@@ -74,19 +74,16 @@ static int RSblock_init(RSblock *blocks, int spec[5], unsigned char *data, unsig
 	int i;
 	RSblock *block;
 	unsigned char *dp, *ep;
-	RS *rs;
 	int el, dl;
 
 	dl = QRspec_rsDataCodes1(spec);
 	el = QRspec_rsEccCodes1(spec);
-	rs = init_rs(8, 0x11d, 0, 1, el, 255 - dl - el);
-	if(rs == NULL) return -1;
 
 	block = blocks;
 	dp = data;
 	ep = ecc;
-	for(i=0; i<QRspec_rsBlockNum1(spec); i++) {
-		RSblock_initBlock(block, dl, dp, el, ep, rs);
+	for(i = 0; i < QRspec_rsBlockNum1(spec); i++) {
+		RSblock_initBlock(block, dl, dp, el, ep);
 		dp += dl;
 		ep += el;
 		block++;
@@ -96,10 +93,8 @@ static int RSblock_init(RSblock *blocks, int spec[5], unsigned char *data, unsig
 
 	dl = QRspec_rsDataCodes2(spec);
 	el = QRspec_rsEccCodes2(spec);
-	rs = init_rs(8, 0x11d, 0, 1, el, 255 - dl - el);
-	if(rs == NULL) return -1;
-	for(i=0; i<QRspec_rsBlockNum2(spec); i++) {
-		RSblock_initBlock(block, dl, dp, el, ep, rs);
+	for(i = 0; i < QRspec_rsBlockNum2(spec); i++) {
+		RSblock_initBlock(block, dl, dp, el, ep);
 		dp += dl;
 		ep += el;
 		block++;
@@ -108,8 +103,8 @@ static int RSblock_init(RSblock *blocks, int spec[5], unsigned char *data, unsig
 	return 0;
 }
 
-__STATIC void QRraw_free(QRRawCode *raw);
-__STATIC QRRawCode *QRraw_new(QRinput *input)
+STATIC_IN_RELEASE void QRraw_free(QRRawCode *raw);
+STATIC_IN_RELEASE QRRawCode *QRraw_new(QRinput *input)
 {
 	QRRawCode *raw;
 	int spec[5], ret;
@@ -159,7 +154,7 @@ __STATIC QRRawCode *QRraw_new(QRinput *input)
  * @param raw raw code.
  * @return code
  */
-__STATIC unsigned char QRraw_getCode(QRRawCode *raw)
+STATIC_IN_RELEASE unsigned char QRraw_getCode(QRRawCode *raw)
 {
 	int col, row;
 	unsigned char ret;
@@ -182,7 +177,7 @@ __STATIC unsigned char QRraw_getCode(QRRawCode *raw)
 	return ret;
 }
 
-__STATIC void QRraw_free(QRRawCode *raw)
+STATIC_IN_RELEASE void QRraw_free(QRRawCode *raw)
 {
 	if(raw != NULL) {
 		free(raw->datacode);
@@ -207,11 +202,10 @@ typedef struct {
 	int count;
 } MQRRawCode;
 
-__STATIC void MQRraw_free(MQRRawCode *raw);
-__STATIC MQRRawCode *MQRraw_new(QRinput *input)
+STATIC_IN_RELEASE void MQRraw_free(MQRRawCode *raw);
+STATIC_IN_RELEASE MQRRawCode *MQRraw_new(QRinput *input)
 {
 	MQRRawCode *raw;
-	RS *rs;
 
 	raw = (MQRRawCode *)malloc(sizeof(MQRRawCode));
 	if(raw == NULL) return NULL;
@@ -238,13 +232,7 @@ __STATIC MQRRawCode *MQRraw_new(QRinput *input)
 		return NULL;
 	}
 
-	rs = init_rs(8, 0x11d, 0, 1, raw->eccLength, 255 - raw->dataLength - raw->eccLength);
-	if(rs == NULL) {
-		MQRraw_free(raw);
-		return NULL;
-	}
-
-	RSblock_initBlock(raw->rsblock, raw->dataLength, raw->datacode, raw->eccLength, raw->ecccode, rs);
+	RSblock_initBlock(raw->rsblock, raw->dataLength, raw->datacode, raw->eccLength, raw->ecccode);
 
 	raw->count = 0;
 
@@ -257,7 +245,7 @@ __STATIC MQRRawCode *MQRraw_new(QRinput *input)
  * @param raw raw code.
  * @return code
  */
-__STATIC unsigned char MQRraw_getCode(MQRRawCode *raw)
+STATIC_IN_RELEASE unsigned char MQRraw_getCode(MQRRawCode *raw)
 {
 	unsigned char ret;
 
@@ -272,7 +260,7 @@ __STATIC unsigned char MQRraw_getCode(MQRRawCode *raw)
 	return ret;
 }
 
-__STATIC void MQRraw_free(MQRRawCode *raw)
+STATIC_IN_RELEASE void MQRraw_free(MQRRawCode *raw)
 {
 	if(raw != NULL) {
 		free(raw->datacode);
@@ -296,12 +284,8 @@ typedef struct {
 	int mqr;
 } FrameFiller;
 
-static FrameFiller *FrameFiller_new(int width, unsigned char *frame, int mqr)
+static void FrameFiller_set(FrameFiller *filler, int width, unsigned char *frame, int mqr)
 {
-	FrameFiller *filler;
-
-	filler = (FrameFiller *)malloc(sizeof(FrameFiller));
-	if(filler == NULL) return NULL;
 	filler->width = width;
 	filler->frame = frame;
 	filler->x = width - 1;
@@ -309,8 +293,6 @@ static FrameFiller *FrameFiller_new(int width, unsigned char *frame, int mqr)
 	filler->dir = -1;
 	filler->bit = -1;
 	filler->mqr = mqr;
-
-	return filler;
 }
 
 static unsigned char *FrameFiller_next(FrameFiller *filler)
@@ -347,15 +329,13 @@ static unsigned char *FrameFiller_next(FrameFiller *filler)
 				y = 9;
 			}
 		}
-	} else {
-		if(y == w) {
-			y = w - 1;
-			x -= 2;
-			filler->dir = -1;
-			if(!filler->mqr && x == 6) {
-				x--;
-				y -= 8;
-			}
+	} else if(y == w) {
+		y = w - 1;
+		x -= 2;
+		filler->dir = -1;
+		if(!filler->mqr && x == 6) {
+			x--;
+			y -= 8;
 		}
 	}
 	if(x < 0 || y < 0) return NULL;
@@ -371,64 +351,52 @@ static unsigned char *FrameFiller_next(FrameFiller *filler)
 }
 
 #ifdef WITH_TESTS
-extern unsigned char *FrameFiller_test(int version)
+unsigned char *FrameFiller_test(int version)
 {
 	int width;
 	unsigned char *frame, *p;
-	FrameFiller *filler;
 	int i, length;
+	FrameFiller filler;
 
 	width = QRspec_getWidth(version);
 	frame = QRspec_newFrame(version);
 	if(frame == NULL) return NULL;
-	filler = FrameFiller_new(width, frame, 0);
-	if(filler == NULL) {
-		free(frame);
-		return NULL;
-	}
+	FrameFiller_set(&filler, width, frame, 0);
 	length = QRspec_getDataLength(version, QR_ECLEVEL_L) * 8
 	       + QRspec_getECCLength(version, QR_ECLEVEL_L) * 8
 		   + QRspec_getRemainder(version);
-	for(i=0; i<length; i++) {
-		p = FrameFiller_next(filler);
+	for(i = 0; i < length; i++) {
+		p = FrameFiller_next(&filler);
 		if(p == NULL) {
-			free(filler);
 			free(frame);
 			return NULL;
 		}
 		*p = (unsigned char)(i & 0x7f) | 0x80;
 	}
-	free(filler);
 	return frame;
 }
 
-extern unsigned char *FrameFiller_testMQR(int version)
+unsigned char *FrameFiller_testMQR(int version)
 {
 	int width;
 	unsigned char *frame, *p;
-	FrameFiller *filler;
 	int i, length;
+	FrameFiller filler;
 
 	width = MQRspec_getWidth(version);
 	frame = MQRspec_newFrame(version);
 	if(frame == NULL) return NULL;
-	filler = FrameFiller_new(width, frame, 1);
-	if(filler == NULL) {
-		free(frame);
-		return NULL;
-	}
+	FrameFiller_set(&filler, width, frame, 1);
 	length = MQRspec_getDataLengthBit(version, QR_ECLEVEL_L)
 	       + MQRspec_getECCLength(version, QR_ECLEVEL_L) * 8;
-	for(i=0; i<length; i++) {
-		p = FrameFiller_next(filler);
+	for(i = 0; i < length; i++) {
+		p = FrameFiller_next(&filler);
 		if(p == NULL) {
 			fprintf(stderr, "Frame filler run over the frame!\n");
-			free(filler);
 			return frame;
 		}
 		*p = (unsigned char)(i & 0x7f) | 0x80;
 	}
-	free(filler);
 	return frame;
 }
 #endif
@@ -438,7 +406,7 @@ extern unsigned char *FrameFiller_testMQR(int version)
  * QR-code encoding
  *****************************************************************************/
 
-__STATIC QRcode *QRcode_new(int version, int width, unsigned char *data)
+STATIC_IN_RELEASE QRcode *QRcode_new(int version, int width, unsigned char *data)
 {
 	QRcode *qrcode;
 
@@ -460,14 +428,14 @@ void QRcode_free(QRcode *qrcode)
 	}
 }
 
-__STATIC QRcode *QRcode_encodeMask(QRinput *input, int mask)
+STATIC_IN_RELEASE QRcode *QRcode_encodeMask(QRinput *input, int mask)
 {
 	int width, version;
 	QRRawCode *raw;
 	unsigned char *frame, *masked, *p, code, bit;
-	FrameFiller *filler;
 	int i, j;
 	QRcode *qrcode = NULL;
+	FrameFiller filler;
 
 	if(input->mqr) {
 		errno = EINVAL;
@@ -492,19 +460,14 @@ __STATIC QRcode *QRcode_encodeMask(QRinput *input, int mask)
 		QRraw_free(raw);
 		return NULL;
 	}
-	filler = FrameFiller_new(width, frame, 0);
-	if(filler == NULL) {
-		QRraw_free(raw);
-		free(frame);
-		return NULL;
-	}
+	FrameFiller_set(&filler, width, frame, 0);
 
-	/* inteleaved data and ecc codes */
-	for(i=0; i<raw->dataLength + raw->eccLength; i++) {
+	/* interleaved data and ecc codes */
+	for(i = 0; i < raw->dataLength + raw->eccLength; i++) {
 		code = QRraw_getCode(raw);
 		bit = 0x80;
-		for(j=0; j<8; j++) {
-			p = FrameFiller_next(filler);
+		for(j = 0; j < 8; j++) {
+			p = FrameFiller_next(&filler);
 			if(p == NULL)  goto EXIT;
 			*p = 0x02 | ((bit & code) != 0);
 			bit = bit >> 1;
@@ -514,8 +477,8 @@ __STATIC QRcode *QRcode_encodeMask(QRinput *input, int mask)
 	raw = NULL;
 	/* remainder bits */
 	j = QRspec_getRemainder(version);
-	for(i=0; i<j; i++) {
-		p = FrameFiller_next(filler);
+	for(i = 0; i < j; i++) {
+		p = FrameFiller_next(&filler);
 		if(p == NULL)  goto EXIT;
 		*p = 0x02;
 	}
@@ -539,19 +502,18 @@ __STATIC QRcode *QRcode_encodeMask(QRinput *input, int mask)
 
 EXIT:
 	QRraw_free(raw);
-	free(filler);
 	free(frame);
 	return qrcode;
 }
 
-__STATIC QRcode *QRcode_encodeMaskMQR(QRinput *input, int mask)
+STATIC_IN_RELEASE QRcode *QRcode_encodeMaskMQR(QRinput *input, int mask)
 {
 	int width, version;
 	MQRRawCode *raw;
 	unsigned char *frame, *masked, *p, code, bit;
-	FrameFiller *filler;
-	int i, j;
+	int i, j, length;
 	QRcode *qrcode = NULL;
+	FrameFiller filler;
 
 	if(!input->mqr) {
 		errno = EINVAL;
@@ -576,39 +538,32 @@ __STATIC QRcode *QRcode_encodeMaskMQR(QRinput *input, int mask)
 		MQRraw_free(raw);
 		return NULL;
 	}
-	filler = FrameFiller_new(width, frame, 1);
-	if(filler == NULL) {
-		MQRraw_free(raw);
-		free(frame);
-		return NULL;
-	}
+	FrameFiller_set(&filler, width, frame, 1);
 
-	/* inteleaved data and ecc codes */
-	for(i=0; i<raw->dataLength + raw->eccLength; i++) {
+	/* interleaved data and ecc codes */
+	for(i = 0; i < raw->dataLength + raw->eccLength; i++) {
 		code = MQRraw_getCode(raw);
+		bit = 0x80;
 		if(raw->oddbits && i == raw->dataLength - 1) {
-			bit = 1 << (raw->oddbits - 1);
-			for(j=0; j<raw->oddbits; j++) {
-				p = FrameFiller_next(filler);
-				if(p == NULL) goto EXIT;
-				*p = 0x02 | ((bit & code) != 0);
-				bit = bit >> 1;
-			}
+			length = raw->oddbits;
 		} else {
-			bit = 0x80;
-			for(j=0; j<8; j++) {
-				p = FrameFiller_next(filler);
-				if(p == NULL) goto EXIT;
-				*p = 0x02 | ((bit & code) != 0);
-				bit = bit >> 1;
-			}
+			length = 8;
+		}
+		for(j = 0; j < length; j++) {
+			p = FrameFiller_next(&filler);
+			if(p == NULL) goto EXIT;
+			*p = 0x02 | ((bit & code) != 0);
+			bit = bit >> 1;
 		}
 	}
 	MQRraw_free(raw);
 	raw = NULL;
 
 	/* masking */
-	if(mask < 0) {
+	if(mask == -2) { // just for debug purpose
+		masked = (unsigned char *)malloc(width * width);
+		memcpy(masked, frame, width * width);
+	} else if(mask < 0) {
 		masked = MMask_mask(version, frame, input->level);
 	} else {
 		masked = MMask_makeMask(version, frame, mask, input->level);
@@ -618,10 +573,12 @@ __STATIC QRcode *QRcode_encodeMaskMQR(QRinput *input, int mask)
 	}
 
 	qrcode = QRcode_new(version, width, masked);
+	if(qrcode == NULL) {
+		free(masked);
+	}
 
 EXIT:
 	MQRraw_free(raw);
-	free(filler);
 	free(frame);
 	return qrcode;
 }
@@ -791,7 +748,7 @@ static unsigned char QRcode_parity(const char *str, int size)
 	unsigned char parity = 0;
 	int i;
 
-	for(i=0; i<size; i++) {
+	for(i = 0; i < size; i++) {
 		parity ^= str[i];
 	}
 
@@ -926,7 +883,5 @@ char *QRcode_APIVersionString(void)
 
 void QRcode_clearCache(void)
 {
-	QRspec_clearCache();
-	MQRspec_clearCache();
-	free_rs_cache();
+	return;
 }
