@@ -2,14 +2,7 @@
 #define __DECODER_H__
 
 #include "../qrencode.h"
-
-typedef struct _DataChunk {
-	QRencodeMode mode;
-	int size;
-	int bits;
-	unsigned char *data;
-	struct _DataChunk *next;
-} DataChunk;
+#include "datachunk.h"
 
 typedef struct {
 	int size;
@@ -18,6 +11,7 @@ typedef struct {
 	int version;
 	QRecLevel level;
 	DataChunk *chunks, *last;
+	int eccResult;
 } QRdata;
 
 struct FormatInfo {
@@ -33,16 +27,16 @@ int QRdata_decodeBitStream(QRdata *qrdata, BitStream *bstream);
 void QRdata_dump(QRdata *data);
 void QRdata_free(QRdata *data);
 
-unsigned int QRcode_decodeVersion(QRcode *code);
+int QRcode_decodeVersion(QRcode *code);
 int QRcode_decodeFormat(QRcode *code, QRecLevel *level, int *mask);
 unsigned char *QRcode_unmask(QRcode *code);
-unsigned char *QRcode_extractBits(QRcode *code, int *length);
+BitStream *QRcode_extractBits(QRcode *code, int *dataLength, int *eccLength);
 QRdata *QRcode_decodeBits(QRcode *code);
 QRdata *QRcode_decode(QRcode *code);
 
 int QRcode_decodeFormatMQR(QRcode *code, int *vesion, QRecLevel *level, int *mask);
 unsigned char *QRcode_unmaskMQR(QRcode *code);
-unsigned char *QRcode_extractBitsMQR(QRcode *code, int *length);
+BitStream *QRcode_extractBitsMQR(QRcode *code, int *dataLength, int *eccLength, int *version, QRecLevel *level);
 QRdata *QRcode_decodeBitsMQR(QRcode *code);
 QRdata *QRcode_decodeMQR(QRcode *code);
 
