@@ -128,7 +128,7 @@ void test_toByte_4bitpadding(void)
 	bstream = BitStream_new();
 	BitStream_appendNum(bstream, 4, 0xb);
 	result = BitStream_toByte(bstream);
-	assert_equal(result[0], 0xb, "incorrect paddings\n");
+	assert_equal(result[0], 0xb0, "incorrect paddings\n");
 	BitStream_free(bstream);
 	free(result);
 
@@ -136,7 +136,7 @@ void test_toByte_4bitpadding(void)
 	BitStream_appendNum(bstream, 12, 0x335);
 	result = BitStream_toByte(bstream);
 	assert_equal(result[0], 0x33, "incorrect paddings\n");
-	assert_equal(result[1], 0x05, "incorrect paddings\n");
+	assert_equal(result[1], 0x50, "incorrect paddings\n");
 	BitStream_free(bstream);
 	free(result);
 
@@ -203,7 +203,36 @@ void test_append(void)
 	BitStream_free(bs2);
 }
 
-int main(void)
+void test_newWithBits(void)
+{
+	BitStream *bstream;
+	unsigned char data[4] = {0, 1, 0, 1};
+
+	testStart("New with bits");
+
+	bstream = BitStream_newWithBits(4, data);
+	assert_equal(bstream->length, 4, "Internal bit length is incorrect.\n");
+	assert_equal(bstream->datasize, 4, "Internal buffer size is incorrect.\n");
+	assert_zero(cmpBin("0101", bstream), "Internal data is incorrect.\n");
+
+	testFinish();
+}
+
+void test_newWithBits_size0(void)
+{
+	BitStream *bstream;
+
+	testStart("New with bits (size = 0)");
+
+	bstream = BitStream_newWithBits(0, NULL);
+	assert_equal(bstream->length, 0, "Internal bit length is incorrect.\n");
+	assert_nonzero(bstream->datasize, "Internal buffer size is incorrect.\n");
+	assert_nonnull(bstream->data, "Internal buffer not allocated.\n");
+
+	testFinish();
+}
+
+int main(int argc, char **argv)
 {
 	test_null();
 	test_num();
@@ -214,6 +243,8 @@ int main(void)
 	test_toByte_4bitpadding();
 	test_size();
 	test_append();
+	test_newWithBits();
+	test_newWithBits_size0();
 
 	report();
 
