@@ -84,7 +84,8 @@ void test_encode_an(int num)
 	ret = memcmp(qrdata->data, data, len);
 	if(ret != 0) {
 		unsigned char *frame, *p;
-		int x,y, c;
+		int x,y,c;
+		int dataLength, eccLength;
 		QRinput *input;
 		QRcode *origcode;
 		BitStream *bstream;
@@ -140,7 +141,8 @@ void test_encode_an(int num)
 
 		snprintf(buf, 256, "monkey-orig-bits-%d.dat", num);
 		fp = fopen(buf, "w");
-		bstream = QRinput_mergeBitStream(input);
+		bstream = BitStream_new();
+		QRinput_mergeBitStream(input, bstream);
 		c = 0;
 		for(x=0; x<bstream->length; x++) {
 			fputc((bstream->data[x]&1)?'1':'0', fp);
@@ -158,7 +160,9 @@ void test_encode_an(int num)
 
 		snprintf(buf, 256, "monkey-result-bits-%d.dat", num);
 		fp = fopen(buf, "w");
-		p = QRcode_extractBits(qrcode, &y);
+		bstream = QRcode_extractBits(qrcode, &dataLength, &eccLength);
+		y = bstream->length;
+		p = bstream->data;
 		c = 0;
 		for(x=0; x<y; x++) {
 			fputc((p[x]&1)?'1':'0', fp);
@@ -171,7 +175,7 @@ void test_encode_an(int num)
 			}
 		}
 		fclose(fp);
-		free(p);
+		BitStream_free(bstream);
 	}
 	QRdata_free(qrdata);
 	QRcode_free(qrcode);
@@ -561,7 +565,5 @@ int main(int argc, char **argv)
 	monkey_encode_kanji(loop);
 	monkey_split_structure(loop);
 
-	QRcode_clearCache();
-
 	return 0;
 }
